#pragma once
#include <stdint.h>
#include <stdbool.h>

// Uncomment to enable debug prints
// #define BREAKPOINT_DEBUG_PRINT



#ifdef BREAKPOINT_DEBUG_PRINT

#include <stdio.h> 
#include <stdlib.h>
#include "gdb-stub.h"
static char debug_print_buffer[1024];
#define gdb_debug_printf(format, args...) do {snprintf(debug_print_buffer, 1024, args);gdb_print(debug_print_buffer);} while (0)
#else
#define gdb_debug_printf(format, args...) while(0)
#endif

void breakpoint_init(void);

void breakpoint_single_step(uint32_t *registers, uint32_t *addr);

void breakpoint_clear_trap(void);

bool breakpoint_insert(uint32_t *addr);

void breakpoint_delete(uint32_t *addr);

// Used by host interrupt routine to give control to GDB
void breakpoint_trap(uint32_t *pc);

// Single step related functions
bool is_mov_pc_instr(uint32_t instr);
bool is_ldm_instr(uint32_t instr);
bool is_ldr_pc_instr(uint32_t instr);
bool is_branch_instr(uint32_t instr);
bool is_sub_pc_instr(uint32_t instr);
bool is_add_pc_instr(uint32_t instr);
bool is_swi_instr(uint32_t instr);
uint32_t handle_mov_pc_instr(uint32_t instr, uint32_t * registers);
uint32_t handle_ldm_instr(uint32_t instr, uint32_t * registers);
uint32_t handle_ldr_pc_instr(uint32_t instr, uint32_t * registers);
uint32_t handle_sub_pc_instr(uint32_t instr, uint32_t * registers);
uint32_t handle_add_pc_instr(uint32_t instr, uint32_t * registers);
uint32_t handle_branch_instr(uint32_t *instr);
uint32_t handle_swi_instr(void);


