#pragma once

#include <stdint.h>
#include <stdbool.h>
#include "defs.h"

// CPSR is defined to be register number 25 by GDB
#define GDB_CPSR_ID 25
// SPSR is masqueraded as floating point status register FPSR
#define GDB_SPSR_ID 24

#define SIGINT 2 // Interrupt - used for giving control to GDB when receiving a interrupt character
#define SIGILL 4 // Illegal instruction
#define SIGTRAP 5 // Tracepoint/breakpoint trap
#define SIGBUS 10 // When entering from ABORT mode

enum regnames {R0, R1, R2, R3, R4, R5, R6, R7, R8, 
               R9, R10, R11, R12, R13, R14, R15, CPSR, SPSR};
typedef enum operating_modes {USER, FIQ, IRQ, SVC, ABORT, UNDEFINED} operating_mode_t;

extern const uint32_t STEP_BREAKPOINT_INSTR;
extern const uint32_t STANDARD_BREAKPOINT_INSTR;
extern const uint32_t INTERRUPT_BREAKPOINT_INSTR;
extern const uint32_t STATIC_BREAKPOINT_INSTR;

/*
 * While we find nice hex chars, build an int.
 * Return number of chars processed.
 */
int hex_to_int(char **ptr, uint32_t *intValue);

/* Convert ch from a hex digit to an int */
uint8_t hex_to_byte(char ch1, char ch2);

char * byte_to_hex(char *ptr, uint8_t c);

char * int_to_hex(char *ptr, uint32_t c);

char * four_bytes_to_hex(char *ptr, uint32_t c);

/* Convert the memory pointed to by mem into hex, placing result in buf.
 * Return a pointer to the last char put in buf (null).
 */
char * mem_to_hex(char *mem, char *buf, int count);

/* Convert the hex array pointed to by buf into binary to be placed in mem
 * return a pointer to the character AFTER the last byte written */
char * hex_to_mem(char *buf, char *mem, int count);

void mem_copy(uint8_t *dest, const uint8_t *source, uint32_t length);

char * string_copy(char *dest, const char *source);

int string_length(const char *str);

bool matches(const char *packet, const char *pattern);

operating_mode_t get_mode(uint32_t spsr);

unsigned get_cpu_id(void);