#include "mutex.h"

#include <stdint.h>

void gdb_init_mutex(volatile uint32_t * mutex)
{
    *mutex = 0;
}

void gdb_lock_mutex(volatile uint32_t * mutex)
{
    __asm__ (
        // save R4
        "push {r4}\n"

        // init delay between lock attempts
        "MOV r4, #1\n"

        // try lock and finish if successful
        "lock:\n"
        "MOV r2, #1\n"
        "SWP r1, r2, [r0]\n"
        "CMP r1, r2\n"
        "BNE locked\n"
        
        // load delay and wait in a loop
        "MOV r3, r4\n"
        "wait:\n"
        "SUB r3, #1\n"
        "CMP r3, #0\n"
        "BNE wait\n"

        // double delay or load 1 if overflown
        "ADD r4, r4\n"
        "CMP r4, #0\n"
        "MOVEQ r4, #1\n"
        
        // repeat lock attempt
        "B lock\n"
        
        "locked:\n"

        // restore R4
        "pop {r4}\n"
    );
}

void gdb_unlock_mutex(volatile uint32_t * mutex)
{
    __asm__ (
        "MOV r2, #0\n"
        "SWP r1, r2, [r0]\n"
    );
}

