#include "packet.h"
#include "serial.h"
#include "misc.h"

static const char hexchars[]="0123456789abcdef";

char * read_packet(char * input_buffer, unsigned buffer_size, unsigned channel)
{
    char *buffer = input_buffer;
    char ch = serial_read_char(channel);
    while (ch != '$')
    {
        ch = serial_read_char(channel);
    }

    bool success = false;
    while (!success)
    {
        while (ch == '$')
        {
            ch = serial_read_char(channel);
        }
        unsigned count = 0;
        char checksum = 0;
        bool dollar_found = false, hash_found = false;
        while (count < buffer_size - 1 && !dollar_found && !hash_found)
        {
            // Is escape charachter
            if (ch == '}')
            {
                checksum += ch;
                // Read next charachter
                ch = serial_read_char(channel);
                checksum += ch;
                // XOR it to get original char
                buffer[count++] = ch ^ 0x20;
                ch = serial_read_char(channel);
            } else if (ch == '$')
            {
                dollar_found = true;
            } else if (ch == '#')
            {
                hash_found = true;
            } else
            {
                checksum += ch;
                buffer[count++] = ch;
                ch = serial_read_char(channel);
            }
            
        }
        // Invalid package
        if (dollar_found) continue;
        // Too long package - drop data and start over
        if (count == buffer_size) continue;
        // Add null-terminator to data
        buffer[count] = 0;

        char cks_1 = serial_read_char(channel);
        char cks_2 = serial_read_char(channel);
        char received_checksum = hex_to_byte(cks_1, cks_2);

        if (checksum != received_checksum)
        {
            serial_write_char('-', channel);   /* failed checksum */
        } else 
        {
            serial_write_char('+', channel);   /* successful transfer */
            success = true;
        }
    }
    return input_buffer;
}

static void put_char(char ch, char *checksum, unsigned channel)
{
    *checksum += ch;
    serial_write_char(ch, channel);
}

/* send the packet in buffer.  */
void put_packet(const char *buffer, unsigned channel)
{

    /*  $<packet info>#<checksum>. */
    do
    {
        serial_write_char('$', channel);
        char checksum = 0;

        for (int i = 0; buffer[i] != 0; i++)
        {
            char ch = buffer[i];
            // * has to be escaped
            if (ch == '*')
            {
                put_char('}', &checksum, channel);
            }
            put_char(ch, &checksum, channel);
            
            // Code below is for run-length encoding
            int n_chars;
            for (n_chars = 0; buffer[n_chars + i + 1] == ch && n_chars < 126; n_chars++); // Max repeat count is 126
            // Check if its worth to run-lenght
            if (n_chars == 1)
            {
                put_char(ch, &checksum, channel);
            } else if (n_chars == 2)
            {
                put_char(ch, &checksum, channel);
                put_char(ch, &checksum, channel);
            } else if (n_chars > 2)
            {
                char encoded_count = 29 + n_chars;
                if (encoded_count == '#')
                {
                    put_char('*', &checksum, channel);
                    put_char(encoded_count - 1, &checksum, channel);
                    put_char(ch, &checksum, channel);
                } else if (encoded_count == '$')
                {
                    put_char('*', &checksum, channel);
                    put_char(encoded_count - 2, &checksum, channel);
                    put_char(ch, &checksum, channel);
                    put_char(ch, &checksum, channel);
                } else
                {
                    put_char('*', &checksum, channel);
                    put_char(encoded_count, &checksum, channel);                    
                }
            }
            i += n_chars;
        }

        serial_write_char('#', channel);
        serial_write_char(hexchars[checksum >> 4], channel);
        serial_write_char(hexchars[checksum & 0xf], channel);
    }
    while (serial_read_char(channel) != '+');
}