#pragma once

#include <stdint.h>

// ============  address regions ===========================

#define RAM_BASE    0x00000000

#define BRAM0_BASE  0xF8000000
#define BRAM1_BASE  0xF9000000
#define BRAM2_BASE  0xFA000000
#define BRAM3_BASE  0xFB000000
#define BRAM4_BASE  0xFC000000
#define BRAM5_BASE  0xFD000000
#define BRAM6_BASE  0xFE000000
#define BRAM7_BASE  0xFF000000

#define TILE_BASE   0xFFFE0000

#define SYS_BASE    0xFFFF0000

#define BRAM_SIZE   16384

// ============  system registers ==========================

#define SYS_INT_STATUS  ((volatile uint32_t*)(SYS_BASE+0x00))
#define SYS_READY       ((volatile uint32_t*)(SYS_BASE+0x10))
#define SYS_CPU_COUNT   ((volatile uint32_t*)(SYS_BASE+0x20))
#define SYS_TICKS       ((volatile uint32_t*)(SYS_BASE+0x30))

#define SYS_OUT_DATA_0    ((volatile uint32_t*)(SYS_BASE+0x100))
#define SYS_IN_DATA_0     ((volatile uint32_t*)(SYS_BASE+0x200))
#define SYS_OUT_DATA_1    ((volatile uint32_t*)(SYS_BASE+0x110))
#define SYS_IN_DATA_1     ((volatile uint32_t*)(SYS_BASE+0x210))
#define SYS_OUT_DATA_2    ((volatile uint32_t*)(SYS_BASE+0x120))
#define SYS_IN_DATA_2     ((volatile uint32_t*)(SYS_BASE+0x220))
#define SYS_OUT_DATA_3    ((volatile uint32_t*)(SYS_BASE+0x130))
#define SYS_IN_DATA_3     ((volatile uint32_t*)(SYS_BASE+0x230))
#define SYS_OUT_DATA_4    ((volatile uint32_t*)(SYS_BASE+0x140))
#define SYS_IN_DATA_4     ((volatile uint32_t*)(SYS_BASE+0x240))
#define SYS_OUT_DATA_5    ((volatile uint32_t*)(SYS_BASE+0x150))
#define SYS_IN_DATA_5     ((volatile uint32_t*)(SYS_BASE+0x250))
#define SYS_OUT_DATA_6    ((volatile uint32_t*)(SYS_BASE+0x160))
#define SYS_IN_DATA_6     ((volatile uint32_t*)(SYS_BASE+0x260))
#define SYS_OUT_DATA_7    ((volatile uint32_t*)(SYS_BASE+0x170))
#define SYS_IN_DATA_7     ((volatile uint32_t*)(SYS_BASE+0x270))
#define SYS_OUT_DATA_8    ((volatile uint32_t*)(SYS_BASE+0x180))
#define SYS_IN_DATA_8     ((volatile uint32_t*)(SYS_BASE+0x280))
#define SYS_OUT_DATA_9    ((volatile uint32_t*)(SYS_BASE+0x190))
#define SYS_IN_DATA_9     ((volatile uint32_t*)(SYS_BASE+0x290))
#define SYS_OUT_DATA_10   ((volatile uint32_t*)(SYS_BASE+0x1a0))
#define SYS_IN_DATA_10    ((volatile uint32_t*)(SYS_BASE+0x2a0))
#define SYS_OUT_DATA_11   ((volatile uint32_t*)(SYS_BASE+0x1b0))
#define SYS_IN_DATA_11    ((volatile uint32_t*)(SYS_BASE+0x2b0))
#define SYS_OUT_DATA_12   ((volatile uint32_t*)(SYS_BASE+0x1c0))
#define SYS_IN_DATA_12    ((volatile uint32_t*)(SYS_BASE+0x2c0))
#define SYS_OUT_DATA_13   ((volatile uint32_t*)(SYS_BASE+0x1d0))
#define SYS_IN_DATA_13    ((volatile uint32_t*)(SYS_BASE+0x2d0))
#define SYS_OUT_DATA_14   ((volatile uint32_t*)(SYS_BASE+0x1e0))
#define SYS_IN_DATA_14    ((volatile uint32_t*)(SYS_BASE+0x2e0))
#define SYS_OUT_DATA_15   ((volatile uint32_t*)(SYS_BASE+0x1f0))
#define SYS_IN_DATA_15    ((volatile uint32_t*)(SYS_BASE+0x2f0))

// ============  tile units ================================

#define TILE_REGS_BASE  (TILE_BASE+0x0000)
#define TIMER0_BASE     (TILE_BASE+0x1000)
#define TIMER1_BASE     (TILE_BASE+0x1100)
#define TIMER2_BASE     (TILE_BASE+0x1200)
#define INT_CTRL0_BASE  (TILE_BASE+0x2000)
#define INT_CTRL1_BASE  (TILE_BASE+0x2040)

// ============  tile registers  ===========================

#define TILEREG_CPUID     ((volatile uint32_t*)(TILE_REGS_BASE+0x000))
#define TILEREG_TILE_X    ((volatile uint32_t*)(TILE_REGS_BASE+0x004))
#define TILEREG_TILE_Y    ((volatile uint32_t*)(TILE_REGS_BASE+0x008))
#define TILEREG_DUMMY     ((volatile uint32_t*)(TILE_REGS_BASE+0x00c))

#define TIMER0_LOAD      ((volatile uint32_t*)(TIMER0_BASE+0x00))
#define TIMER0_VALUE     ((volatile uint32_t*)(TIMER0_BASE+0x04))
#define TIMER0_CTRL      ((volatile uint32_t*)(TIMER0_BASE+0x08))
#define TIMER0_CLR       ((volatile uint32_t*)(TIMER0_BASE+0x0c))

#define TIMER1_LOAD      ((volatile uint32_t*)(TIMER1_BASE+0x00))
#define TIMER1_VALUE     ((volatile uint32_t*)(TIMER1_BASE+0x04))
#define TIMER1_CTRL      ((volatile uint32_t*)(TIMER1_BASE+0x08))
#define TIMER1_CLR       ((volatile uint32_t*)(TIMER1_BASE+0x0c))

#define TIMER2_LOAD      ((volatile uint32_t*)(TIMER2_BASE+0x00))
#define TIMER2_VALUE     ((volatile uint32_t*)(TIMER2_BASE+0x04))
#define TIMER2_CTRL      ((volatile uint32_t*)(TIMER2_BASE+0x08))
#define TIMER2_CLR       ((volatile uint32_t*)(TIMER2_BASE+0x0c))

#define IC0_IRQ_STATUS      ((volatile uint32_t*)(INT_CTRL0_BASE+0x00))
#define IC0_IRQ_RAWSTAT     ((volatile uint32_t*)(INT_CTRL0_BASE+0x04))
#define IC0_IRQ_ENABLESET   ((volatile uint32_t*)(INT_CTRL0_BASE+0x08)) 
#define IC0_IRQ_ENABLECLR   ((volatile uint32_t*)(INT_CTRL0_BASE+0x0c)) 
#define IC0_INT_SOFTSET     ((volatile uint32_t*)(INT_CTRL0_BASE+0x10))
#define IC0_INT_SOFTCLEAR   ((volatile uint32_t*)(INT_CTRL0_BASE+0x14))
#define IC0_FIRQ_STATUS     ((volatile uint32_t*)(INT_CTRL0_BASE+0x20))  
#define IC0_FIRQ_RAWSTAT    ((volatile uint32_t*)(INT_CTRL0_BASE+0x24))  
#define IC0_FIRQ_ENABLESET  ((volatile uint32_t*)(INT_CTRL0_BASE+0x28))  
#define IC0_FIRQ_ENABLECLR  ((volatile uint32_t*)(INT_CTRL0_BASE+0x2c)) 

#define IC1_IRQ_STATUS      ((volatile uint32_t*)(INT_CTRL1_BASE+0x00))
#define IC1_IRQ_RAWSTAT     ((volatile uint32_t*)(INT_CTRL1_BASE+0x04))
#define IC1_IRQ_ENABLESET   ((volatile uint32_t*)(INT_CTRL1_BASE+0x08)) 
#define IC1_IRQ_ENABLECLR   ((volatile uint32_t*)(INT_CTRL1_BASE+0x0c)) 
#define IC1_INT_SOFTSET     ((volatile uint32_t*)(INT_CTRL1_BASE+0x10))
#define IC1_INT_SOFTCLEAR   ((volatile uint32_t*)(INT_CTRL1_BASE+0x14))
#define IC1_FIRQ_STATUS     ((volatile uint32_t*)(INT_CTRL1_BASE+0x20))  
#define IC1_FIRQ_RAWSTAT    ((volatile uint32_t*)(INT_CTRL1_BASE+0x24))  
#define IC1_FIRQ_ENABLESET  ((volatile uint32_t*)(INT_CTRL1_BASE+0x28))  
#define IC1_FIRQ_ENABLECLR  ((volatile uint32_t*)(INT_CTRL1_BASE+0x2c)) 

// ============  timer ctrl flags ==========================

#define TIMER_CTRL_ENABLE (1<<7)
#define TIMER_CTRL_PERIODIC (1<<6)
#define TIMER_CTRL_SCALE_256 0
#define TIMER_CTRL_SCALE_16 4
#define TIMER_CTRL_SCALE_1 8

// ============  interrupt bits ==========================

#define INT_MASK_SOFT (1 << 0)

#define INT_MASK_TIMER0 (1 << 1)
#define INT_MASK_TIMER1 (1 << 2)
#define INT_MASK_TIMER2 (1 << 3)

#define INT_MASK_HOST_0 (1 << 16)
#define INT_MASK_HOST_1 (1 << 17)
#define INT_MASK_HOST_2 (1 << 18)
#define INT_MASK_HOST_3 (1 << 19)

#define INT_MASK_HOST_4 (1 << 20)
#define INT_MASK_HOST_5 (1 << 21)
#define INT_MASK_HOST_6 (1 << 22)
#define INT_MASK_HOST_7 (1 << 23)

#define INT_MASK_HOST_8 (1 << 24)
#define INT_MASK_HOST_9 (1 << 25)
#define INT_MASK_HOST_10 (1 << 26)
#define INT_MASK_HOST_11 (1 << 27)

#define INT_MASK_HOST_12 (1 << 28)
#define INT_MASK_HOST_13 (1 << 29)
#define INT_MASK_HOST_14 (1 << 30)
#define INT_MASK_HOST_15 (1 << 31)
