.global vector
.global gdb_irq_handler_program
.global gdb_undef_instr_handler_program

vector:
    b reset_handler
    b gdb_undef_instr_handler
    b swi_handler
    b . // Prefetch abort
    b . // Data abort
    b . // Reserved
    b gdb_irq_handler
    b . // FIQ

gdb_irq_handler_program:
irq_handler:
    // Save regs
    push {r0-r3,r12,lr}

    // Jump to C function
    bl irq_c_handler

    // Restore regs
    pop {r0-r3,r12,lr}

    // Return
    subs pc, lr, #4

gdb_undef_instr_handler_program:
undef_instr_handler:
    // Save regs
    push {r0-r3,r12,lr}

    // Jump to C function
    bl undef_instr_c_handler

    // Restore regs
    pop {r0-r3,r12,lr}

    // Return
    movs pc, lr

swi_handler:
    // Save regs
    push {r0-r3,r12,lr}

    mov r0, lr
    // Jump to C function
    bl swi_c_handler

    // Restore regs
    pop {r0-r3,r12,lr}

    // Return
    movs pc, lr

reset_handler:
    // Load stack top to r0
    ldr r0, =stack_top

    // load cpu number * 1MB
    ldr r1, =0xFFFE0000
    ldr r1, [r1]
    lsl r1, r1, #20

    // adjust stack position depending on CPU
    sub r0, r0, r1

    // Set stack - 128kB
    mov sp, r0
    sub r0, r0, #0x20000	

    // Change to IRQ mode
    msr cpsr, #0xD2
    nop 
        
    // Set stack - 128kB
    mov sp, r0
    sub r0, r0, #0x20000 

    // Change to UND mode
    msr cpsr, #0xDB
    nop

    // Set stack - 128kB
    mov sp, r0
    sub r0, r0, #0x20000

    // Go to user mode and enable IRQ
    msr cpsr, #0x50
    nop

    // Set stack
    mov sp, r0

    // Launch main
    bl main

    b .
