Toolchain setup for Barrelfish on SHMAC
=========

This guide explains how to setup the needed GCC toolchain needed for compiling Barrelfish to SHMAC. Note that the steps below are based on a setup on OSX 10.9. The guide will use build ``Binutils 2.24``, ``Newlib 2.1.0`` and ``GCC 4.8.2`` from source. Note: Newlib is not required for building Barrelfish.

Software dependencies
----
* GHC 7.6.3 (use haskell-platform package)
* ghc-paths
* GMP 4.2+
* MPFR 2.4.0+
* MPC 0.8.0+

Install dependencies from Homebrew (or your favorite package manager):
```sh
brew install haskell-platform gmp mpfr libmpc
cabal install ghc-paths
```
(Cabal is the Haskell package manager).

Setup directory stucture
----
Create the following folder structure:
* shmac-tools/source/
* shmac-tools/install/
* shmac-tools/build/

Download source code
----
Download the source code for [`bin-utils`](http://ftp.gnu.org/gnu/binutils/binutils-2.24.tar.gz), [`newlib`](http://goo.gl/GqxcsP) and [`gcc`](http://ftp.gnu.org/gnu/gcc/gcc-4.8.2/gcc-4.8.2.tar.gz). Extract the archives into the the **shmac-tools/source** folder.

Build binutils
----
Navigate to the **shmac-tools/build/binutils-2.24** folder and execute the following commands:
```sh
CFLAGS="-O2" CXXFLAGS="-O2" ../../source/binutils-2.24/configure --prefix=`pwd`/../../install/ --target=armv3-none-eabi --disable-werror
```

Then run make:
```sh
make install
```

Build bootstrap GCC
----
Start off by defining a new target in the **shmac-tools/source/gcc-4.8.2/libgcc/config.host** file.

IMPORTANT: Make sure to add the following segment above the target definition for ``arm*-*-eabi*``.

```makefile
armv3-none-eabi)       # SHMAC ARMV3
    tmake_file="${tmake_file} arm/t-arm arm/t-elf t-fixedpoint-gnu-prefix"
    tm_file="$tm_file arm/bpabi-lib.h"
    tmake_file="${tmake_file} arm/t-bpabi-armv3"
    extra_parts="crtbegin.o crtend.o crti.o crtn.o"
    tmake_file="$tmake_file t-softfp-sfdf t-softfp-excl arm/t-softfp t-softfp"
    unwind_header=config/arm/unwind-arm.h
    ;;  
```
Create the file **shmac-tools/source/gcc-4.8.2/libgcc/config/arm/t-bpabi-armv3** with content:
```makefile
# Add the bpabi.S functions.
LIB1ASMFUNCS += _aeabi_lcmp _aeabi_ulcmp _aeabi_ldivmod _aeabi_uldivmod

# Add the BPABI C functions.
LIB2ADD += $(srcdir)/config/arm/bpabi.c \
       $(srcdir)/config/arm/unaligned-funcs.c

LIB2ADD_ST += $(srcdir)/config/arm/fp16.c

LIB2ADDEH = $(srcdir)/config/arm/unwind-arm.c \
  $(srcdir)/config/arm/pr-support.c $(srcdir)/unwind-c.c

# Add the BPABI names.
SHLIB_MAPFILES += $(srcdir)/config/arm/libgcc-bpabi.ver

# On ARM, specifying -fnon-call-exceptions will needlessly pull in
# the unwinder in simple programs which use 64-bit division.  Omitting
# the option is safe.
LIB2_DIVMOD_EXCEPTION_FLAGS := -fno-exceptions
```

Navigate into the **shmac-tools/build/gcc-4.8.2** folder and execute the following configure command:
```sh
CFLAGS="-O2" CXXFLAGS="-O2" CFLAGS_FOR_TARGET="-mno-thumb-interwork -march=armv3" ../../source/gcc-4.8.2/configure --target=armv3-none-eabi --prefix=`pwd`/../../install --disable-thread --with-multilib-list="armv3" --enable-languages="c" --disable-interwork --disable-nls --disable-shared --disable-libssp --with-arch=armv3 --disable-sjlj-exceptions --with-newlib --without-headers
```
You are now ready to build and install GCC:
```sh
make all-gcc
make all-target-libgcc
make install-gcc install-target-libgcc
```

Make sure to add the just built binaries to the path (they are needed for compiling newlib).

Build newlib
----
The ARM target contains assembly files with unsupported instructions. Theese files has to be replaced with the standard C implemenations.

1. Delete the files ``strcmp.S`` and ``strcpy.c`` from **shmac-tools/source/newlib-2.1.0/newlib/libc/machine/arm/**.
2. Copy the files ``strcmp.c`` and ``strcpy.c`` from **shmac-tools/source/newlib-2.1.0/newlib/libc/string** into **shmac-tools/source/newlib-2.1.0/newlib/libc/machine/arm/**.
3. Modify the **shmac-tools/source/newlib-2.1.0/newlib/libc/machine/arm/Makefile.am** to refer to ``strcmp.c`` instead of ``strcmp.S``. Do the same for **shmac-tools/source/newlib-2.1.0/newlib/libc/machine/arm/Makefile.in**. In addition, modify the compilation definition for ``strcmp.S`` to:
                                                                                    
```sh
lib_a-strcmp.o: strcmp.c
    $(CC) $(DEFS) $(DEFAULT_INCLUDES) $(INCLUDES) $(AM_CPPFLAGS) $(CPPFLAGS) $(lib_a_CFLAGS) $(CFLAGS) -c -o lib_a-strcmp.o `test -f 'strcmp.c' || echo '$(srcdir)/'`strcmp.c

lib_a-strcmp.obj: strcmp.c
    $(CC) $(DEFS) $(DEFAULT_INCLUDES) $(INCLUDES) $(AM_CPPFLAGS) $(CPPFLAGS) $(lib_a_CFLAGS) $(CFLAGS) -c -o lib_a-strcmp.obj `if test -f 'strcmp.c'; then $(CYGPATH_W) 'strcmp.c'; else $(CYGPATH_W) '$(srcdir)/strcmp.c'; fi`
```

Navigate to **shmac-tools/build/newlib-2.1.0/** and execute the following commmand to configure newlib:
```sh
CFLAGS="-O2" CXXFLAGS="-O2" CFLAGS_FOR_TARGET="-mno-thumb-interwork -march=armv3" ../../source/newlib-2.1.0/configure --target=armv3-none-eabi --prefix=`pwd`/../../install --disable-libssp --disable-newlib-supplied-syscalls --disable-interwork --disable-multilib --with-gnu-as --with-gnu-ld --disable-nls --with-arch=armv3
```

Then build it:
```sh
make all-target-newlib
make install-target-newlib
```

Build GCC
----
```sh
CFLAGS="-O2" CXXFLAGS="-O2" CFLAGS_FOR_TARGET="-mno-thumb-interwork -march=armv3" ../../source/gcc-4.8.2/configure --target=armv3-none-eabi --prefix=`pwd`/../../install  --disable-thread --with-multilib-list="armv3" --enable-languages="c" --disable-interwork --disable-nls --disable-shared --disable-libssp --with-arch=armv3 --disable-sjlj-exceptions --with-newlib
```

```sh
make all
make install
```

Add the resulting binaries in **shmac-tools/install** to your path.

Note: You will have to link with gcc due to an unknown error.

Aquire the Barrelfish for SHMAC source code
----
Contact either Benjamin Bjørnseth or Bjørn C Seime for details.

Build Barrelfish
----
Create the following folder structure:
* barrelfish/source/
* barrelfish/build/

Navigate to the **barrelfish/build/** folder and execute the following command:
```sh
../source/hake/hake.sh -s ../source/ -a shmac
make
```

You should now have a working setup.